@@ -594,7 +594,7 @@ static const canon_cap_t canon_model_capabilities[] =
     842, 17*72,       /* max paper width and height */
     10, 10, 15, 15,    /*border_left, border_right, border_top, border_bottom */
     &canon_PIXMA_iP4000_slotlist,
-    CANON_CAP_STD0|CANON_CAP_DUPLEX|CANON_CAP_r|CANON_CAP_px|CANON_CAP_I,0x64,control_cmd_PIXMA_iP4000,  /*features */
+    CANON_CAP_STD0|CANON_CAP_DUPLEX|CANON_CAP_r|CANON_CAP_px|CANON_CAP_I,0x61,control_cmd_MULTIPASS_MP150,  /*features */
     &canon_PIXMA_iP5300_modelist,
     &canon_PIXMA_iP4000_paperlist,
     NULL,
@@ -613,6 +613,18 @@ static const canon_cap_t canon_model_capabilities[] =
     iP4200_hue_adjustment,
     iP4200_sat_adjustment
   },
+  { /* Canon PIXMA iP6700 */
+    "PIXMA iP6000", 3,          /*model, model_id*/
+    842, 17*72,       /* max paper width and height */
+    10, 10, 15, 15,    /*border_left, border_right, border_top, border_bottom */
+    &canon_PIXMA_iP4000_slotlist,
+    CANON_CAP_STD0|CANON_CAP_DUPLEX|CANON_CAP_r|CANON_CAP_px,0x64,control_cmd_PIXMA_iP4000,  /*features */
+    &canon_PIXMA_iP6000_modelist,
+    &canon_PIXMA_iP4000_paperlist,
+    NULL,
+    NULL,
+    NULL
+  },
   { /* Canon PIXMA iP6700 */
     "PIXMA iP6700", 3,          /*model, model_id*/
     842, 17*72,       /* max paper width and height */
@@ -637,6 +649,18 @@ static const canon_cap_t canon_model_capabilities[] =
     NULL,
     NULL
   },
+  { /* Canon PIXMA MP520 */
+    "PIXMA MP520", 3,          /*model, model_id*/
+    842, 17*72,       /* max paper width and height */
+    10, 10, 15, 15,    /*border_left, border_right, border_top, border_bottom */
+    &canon_PIXMA_iP4000_slotlist,
+    CANON_CAP_STD0|CANON_CAP_DUPLEX|CANON_CAP_r|CANON_CAP_px|CANON_CAP_I,0x61,control_cmd_PIXMA_iP4000,  /*features */
+    &canon_PIXMA_iX5000_modelist,
+    &canon_PIXMA_iP4000_paperlist,
+    NULL,
+    NULL,
+    NULL
+  },
   { /* Canon PIXMA Pro9500 */
     "PIXMA Pro9500", 3,          /*model, model_id*/
     842, 17*72,       /* max paper width and height */
