@@ -1,5 +1,5 @@
 /*
- * "$Id: sequence.c,v 1.23 2006/09/12 19:03:33 easysw Exp $"
+ * "$Id: sequence.c,v 1.25 2008/01/21 23:19:41 rlk Exp $"
  *
  *   Sequence data type.  This type is designed to be derived from by
  *   the curve and dither matrix types.
@@ -140,6 +140,24 @@ stp_sequence_copy(stp_sequence_t *dest, const stp_sequence_t *source)
   memcpy(dest->data, source->data, (sizeof(double) * source->size));
 }
 
+void
+stp_sequence_reverse(stp_sequence_t *dest, const stp_sequence_t *source)
+{
+  int i;
+  check_sequence(dest);
+  check_sequence(source);
+
+  dest->recompute_range = source->recompute_range;
+  dest->blo = source->blo;
+  dest->bhi = source->bhi;
+  dest->rlo = source->rlo;
+  dest->rhi = source->rhi;
+  dest->size = source->size;
+  dest->data = stp_zalloc(sizeof(double) * source->size);
+  for (i = 0; i < source->size; i++)
+    dest->data[i] = source->data[source->size - i - 1];
+}
+
 stp_sequence_t *
 stp_sequence_create_copy(const stp_sequence_t *sequence)
 {
@@ -150,6 +168,16 @@ stp_sequence_create_copy(const stp_sequence_t *sequence)
   return ret;
 }
 
+stp_sequence_t *
+stp_sequence_create_reverse(const stp_sequence_t *sequence)
+{
+  stp_sequence_t *ret;
+  check_sequence(sequence);
+  ret = stp_sequence_create();
+  stp_sequence_reverse(ret, sequence);
+  return ret;
+}
+
 int
 stp_sequence_set_bounds(stp_sequence_t *sequence, double low, double high)
 {
@@ -478,14 +506,13 @@ stp_sequence_set_##name##_data(stp_sequence_t *sequence,                     \
                                size_t count, const t *data)                  \
 {									     \
   int i;								     \
-  check_sequence(sequence);						             \
+  check_sequence(sequence);					             \
   if (count < 2)							     \
     return 0;								     \
 									     \
   /* Validate the data before we commit to it. */			     \
   for (i = 0; i < count; i++)						     \
-    if (! isfinite(data[i]) ||                                                 \
-        data[i] < sequence->blo ||                                           \
+    if (data[i] < sequence->blo ||                                           \
         data[i] > sequence->bhi)                                             \
       return 0;								     \
   stp_sequence_set_size(sequence, count);                                    \
@@ -494,7 +521,27 @@ stp_sequence_set_##name##_data(stp_sequence_t *sequence,                     \
   return 1;								     \
 }
 
-DEFINE_DATA_SETTER(float, float)
+int
+stp_sequence_set_float_data(stp_sequence_t *sequence,
+			    size_t count, const float *data)
+{
+  int i;
+  check_sequence(sequence);
+  if (count < 2)
+    return 0;
+
+  /* Validate the data before we commit to it. */
+  for (i = 0; i < count; i++)
+    if (! isfinite(data[i]) ||
+        data[i] < sequence->blo ||
+        data[i] > sequence->bhi)
+      return 0;
+  stp_sequence_set_size(sequence, count);
+  for (i = 0; i < count; i++)
+    stp_sequence_set_point(sequence, i, (double) data[i]);
+  return 1;
+}
+
 DEFINE_DATA_SETTER(long, long)
 DEFINE_DATA_SETTER(unsigned long, ulong)
 DEFINE_DATA_SETTER(int, int)
